@@ -1,7 +1,7 @@
 /****************************************************************************
    Dino - A simple pattern based MIDI sequencer
    
-   Copyright (C) 2006  Lars Luthman <larsl@users.sourceforge.net>
+   Copyright (C) 2006 - 2010  Lars Luthman <lars.luthman@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -582,7 +582,9 @@ bool NoteEditor::on_expose_event(GdkEventExpose* event) {
   // draw box for editing note velocity
   if (m_drag_operation == DragChangingNoteVelocity) {
     iter = m_pat->find_note(m_drag_step, m_drag_note);
-    draw_velocity_box(iter, m_selection.find(iter) != m_selection.end());
+    // XXX - Why is this needed?
+    if (iter != m_pat->notes_end())
+      draw_velocity_box(iter, m_selection.find(iter) != m_selection.end());
   }
   
   // draw outline
